<?php

namespace SM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class ApropriacaoRecursosFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getApropAgendamento() 
    {
        $sql = "SELECT
                    HORADEC_HORACHAR(APRO.QTHORASAPROPRIADAS)       AS QTHORAS,
                    APRO.NRAPROPAGENDAMENTO,
                    APRO.NRRECURSOSAGENDCONT,
                    APRO.NRPARCNEGRECURSO,
                    RECU.NMPRINCIPALPARC                            AS NMPARCNEGRECURSO,
                    TO_CHAR(APRO.DTAPROPRIACAO, 'DD/MM/YYYY')       AS DTAPROPRIACAO,
                    APRO.IDSTATUSAPROPRIACAO,
                    APRO.QTHORASAPROPRIADAS,
                    APRO.NRPARCINTERLOCUTOR,
                    APRO.NRORG,
                    APRO.IDATIVO,
                    APRO.DTINCLUSAO,
                    APRO.NRORGINCLUSAO,
                    APRO.CDOPERINCLUSAO,
                    APRO.DTULTATU,
                    APRO.NRORGULTATU,
                    APRO.CDOPERULTATU,
                    NVL(ESCALA.HRINICIAL, EQUIPE.HRPRIMEIRAENTRADA) AS HRPRIMEIRAENTRADA,
                    EQUIPE.HRPRIMEIRASAIDA,
                    EQUIPE.HRSEGUNDAENTRADA,
                    NVL(ESCALA.HRFINAL, EQUIPE.HRSEGUNDASAIDA)      AS HRSEGUNDASAIDA,
                    EQUIPE.NRTEMPOTOLERANCIA,
                    AGEND.NRSEQCONTRATO,
                    AGEND.NRAGENDRECURSOCONT,
                    CLI.NRPARCNEGOCIO                               AS NRPARCCLIENTE,
                    CONT.NRCONTRATOFCT,
                    CONT.NRCONTRATOFCT
                    || ' - '
                    || CONT.DSREFERENCIACONT                        AS AGENDAMENTO,
                    CLI.NMPRINCIPALPARC                             AS CLIENTE,
                    ITEM.NRITEMOCONTRATO,
                    ITEM.CDSERVORCA,
                    SERVICO.NMSERVORCA
                FROM
                        FCT_APROPAGENDAMENTO APRO
                    INNER JOIN PARCNEGOCIO                 RECU ON APRO.NRPARCNEGRECURSO = RECU.NRPARCNEGOCIO
                                                AND APRO.NRORG = RECU.NRORG
                    LEFT JOIN FCT_RECURSOSAGENDCONT       EQUIPE ON APRO.NRRECURSOSAGENDCONT = EQUIPE.NRRECURSOSAGENDCONT
                                                            AND APRO.NRORG = EQUIPE.NRORG
                    LEFT JOIN FCT_ESCALARECURSOSAGENDCONT ESCALA ON APRO.NRRECURSOSAGENDCONT = ESCALA.NRRECURSOSAGENDCONT
                                                                    AND APRO.NRORG = ESCALA.NRORG
                                                                    AND DTAPROPRIACAO BETWEEN ESCALA.DTESCALAENTRADA AND ESCALA.DTESCALASAIDA
                    LEFT JOIN FCT_AGENDRECURSOCONT        AGEND ON EQUIPE.NRAGENDRECURSOCONT = AGEND.NRAGENDRECURSOCONT
                                                            AND EQUIPE.NRORG = AGEND.NRORG
                    LEFT JOIN FCT_CONTRATO                CONT ON AGEND.NRSEQCONTRATO = CONT.NRSEQCONTRATO
                                                AND AGEND.NRORG = CONT.NRORG
                    LEFT JOIN FCT_ITEMCONTRATO            ITEM ON ITEM.NRSEQCONTRATO = CONT.NRSEQCONTRATO
                                                AND ITEM.NRORG = CONT.NRORG
                    INNER JOIN FCT_SERVICOORCA          SERVICO ON ITEM.CDSERVORCA = SERVICO.CDSERVORCA
                                                AND ITEM.NRORG = CONT.NRORG
                    LEFT JOIN PARCNEGOCIO                 CLI ON CONT.NRPARCNEGOCIO = CLI.NRPARCNEGOCIO
                                                AND CONT.NRORG = CLI.NRORG ORDER BY APRO.DTAPROPRIACAO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getApontamentoApropAgen($parametros)
    {
        $sql = "SELECT
                    APONT.NRAPONTAMENTOAPROPAGEN,
                    APONT.NRAPROPAGENDAMENTO,
                    APONT.IDTIPOAPONTAMENTO,
                    APONT.IDMOTIVOSAIDA,
                    APONT.DTHRINICIOAPONT,
                    APONT.DTHRFIMCIOAPONT,
                    APONT.DTHRINICIOAPONTCDA,
                    APONT.DTHRFIMCIOAPONTCDA,
                    APONT.QTTEMPOAPONT,
                    APONT.DSOBSAPONTAMENTO,
                    APONT.NRORG,
                    APONT.IDATIVO,
                    APONT.DTINCLUSAO,
                    APONT.DTULTATU,
                    APONT.NRORGINCLUSAO,
                    APONT.CDOPERINCLUSAO,
                    APONT.NRORGULTATU,
                    APONT.CDOPERULTATU,
                    TO_CHAR(DTHRINICIOAPONT, 'HH24:MI') AS DTINICIAL,
                    TO_CHAR(DTHRFIMCIOAPONT, 'HH24:MI') AS DTFIM,
                    HORADEC_HORACHAR(QTTEMPOAPONT)      AS QTTEMPOAPONTADO
                FROM
                    FCT_APONTAMENTOAPROPAGEN APONT
                WHERE
                    NRAPROPAGENDAMENTO = :NRAPROPAGENDAMENTO
                    AND NRORG = :NRORG";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function insertApontamentoApropAgen($parametros)
    {
        $sql = "INSERT INTO FCT_APONTAMENTOAPROPAGEN ( NRAPONTAMENTOAPROPAGEN, NRAPROPAGENDAMENTO, IDTIPOAPONTAMENTO, IDMOTIVOSAIDA, DTHRINICIOAPONTCDA, DTHRFIMCIOAPONTCDA, QTTEMPOAPONT, DSOBSAPONTAMENTO, NRORG, IDATIVO, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
                    VALUES ( :NRAPONTAMENTOAPROPAGEN, :NRAPROPAGENDAMENTO, :IDTIPOAPONTAMENTO, :IDMOTIVOSAIDA, :DTHRINICIOAPONTCDA, :DTHRFIMCIOAPONTCDA, :QTTEMPOAPONT, :DSOBSAPONTAMENTO, :NRORG, :IDATIVO, SYSDATE, :NRORGINCLUSAO, :CDOPERINCLUSAO)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function updateApontamentoApropAgen($parametros)
    {
        $sql = "UPDATE FCT_APONTAMENTOAPROPAGEN 
                    SET IDTIPOAPONTAMENTO   = :IDTIPOAPONTAMENTO,
                        IDMOTIVOSAIDA       = :IDMOTIVOSAIDA,
                        DTHRINICIOAPONTCDA  = :DTHRINICIOAPONTCDA,
                        DTHRFIMCIOAPONTCDA  = :DTHRFIMCIOAPONTCDA,
                        QTTEMPOAPONT        = :QTTEMPOAPONT,
                        DSOBSAPONTAMENTO    = :DSOBSAPONTAMENTO,
                        IDATIVO             = :IDATIVO,
                        DTULTATU            = SYSDATE,
                        NRORGULTATU         = :NRORGULTATU,
                        CDOPERULTATU        = :CDOPERULTATU
                    WHERE NRAPONTAMENTOAPROPAGEN = :NRAPONTAMENTOAPROPAGEN AND NRORG = :NRORG";
                    
        $this->connection->executeStatement($sql, $parametros);
    }

    public function deleteApontamento($parametros)
    {
        $sql = "DELETE FROM FCT_APONTAMENTOAPROPAGEN WHERE NRAPONTAMENTOAPROPAGEN IN ($parametros)";

        $this->connection->executeStatement($sql);
    }
}