import { Utilities } from '@/util/Utilities';
import {
    Button, Modal, ModalService, Select, Tabs, Textarea, TextInput,
    Time, Text,
    Form,
    LoadingService,
    DialogService,
} from '@zeedhi/common';
import { Http, IDictionary, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class ApropriacaoRecursosController {
    private visibilidadeGridApropriacao: boolean = true;

    private currentRow : any = {};

    private rowGridApontamento : any = {};

    private modal!: Modal;

    public tituloModalApontamento : string = '';

    public async abrirModalAddApontamento() {
        this.modal = await ModalService.createFromJson('apropriacaoRecursos/modal/modalApontamento', true);
        this.modal.show();
        setTimeout(() => {
            Metadata.getInstance<Text>('title-modal-apontamento').text = 'Novo Apontamento';
            Metadata.getInstance<TextInput>('form-modal-apontamento-NRAPONTAMENTOAPROPAGEN').value = 'NOVO_APONTAMENTO';
        }, 100);
    }

    public async abrirModalEditApontamento({ row }: any) {
        this.rowGridApontamento = row;
        this.modal = await ModalService.createFromJson('apropriacaoRecursos/modal/modalApontamento', true);
        this.modal.show();
        await this.carregarDadosModalApontamento();
    }

    public fecharModalApontamento() {
        this.modal.hide();
        this.modal?.destroy();
    }

    public get visibilidadeFrameApropriacao() {
        return !this.visibilidadeGridApropriacao;
    }

    public openFilter({ component }: any) {
        const filterButton = Metadata.getInstance<Button>(`${component.name}_filterButton`);
        filterButton.click({} as Event, {} as HTMLElement);
    }

    public abrirFrameApropriacao({ row }: any) {
        this.visibilidadeGridApropriacao = false;
        this.currentRow = row;
        this.changeActiveTab();
    }

    public ativaTabControleApontamento() {
        const setOfTabs = Metadata.getInstance<Tabs>('tabs-apropriacao-recursos');
        setOfTabs.activeTab = 0;
    }
    
    public voltarGridApropriacao() {
        this.visibilidadeGridApropriacao = true;
        this.ativaTabControleApontamento();
    }

    public get tituloFrame() : string {
        return `${this.currentRow.DTAPROPRIACAO} - ${this.currentRow.NRCONTRATOFCT}`;
    }

    public changeActiveTab(): void {
        const tabs = Metadata.getInstance<Tabs>('tabs-apropriacao-recursos');
        const activeTab = tabs.tabs[tabs.activeTab];

        switch (activeTab.name) {
            case 'tab_controleApontamento':
                this.carregarDadosControleApontamento();
                break;
            case 'tab_horasNaoFaturadas':
                // this.setFilterGridHorasNaoFaturadas();
                break;
            case 'tab_apontamento':
                setTimeout(() => {
                    this.setFilterGridApontamentos();
                }, 100);
                break;
            default:
                // Do nothing
                break;
        }
    }

    public carregarDadosControleApontamento() {
        Metadata.getInstance<TextInput>('form-tab-controle-apontamento-descricao-NRAPROPAGENDAMENTO').value = this.currentRow.NRAPROPAGENDAMENTO;
        Metadata.getInstance<TextInput>('form-tab-controle-apontamento-descricao-CLIENTE').value = this.currentRow.CLIENTE;
        Metadata.getInstance<Textarea>('form-tab-controle-apontamento-descricao-AGENDAMENTO').value = this.currentRow.AGENDAMENTO;
        Metadata.getInstance<TextInput>('form-tab-controle-apontamento-descricao-NMSERVORCA').value = this.currentRow.NMSERVORCA;
        Metadata.getInstance<TextInput>('form-tab-controle-apontamento-descricao-NMPARCNEGRECURSO').value = this.currentRow.NMPARCNEGRECURSO;
        Metadata.getInstance<Select>('form-tab-controle-apontamento-descricao-IDSTATUSAPROPRIACAO').value = this.currentRow.IDSTATUSAPROPRIACAO;
        Metadata.getInstance<TextInput>('form-tab-controle-apontamento-descricao-DTAPROPRIACAO').value = this.currentRow.DTAPROPRIACAO;
        Metadata.getInstance<TextInput>('form-tab-controle-apontamento-descricao-QTHORAS').value = this.currentRow.QTHORAS;
        Metadata.getInstance<TextInput>('form-tab-controle-apontamento-descricao-QTHRDESLOCAMENTOFAT').value = this.currentRow.QTHRDESLOCAMENTOFAT;
        Metadata.getInstance<TextInput>('form-tab-controle-apontamento-descricao-QTHRATIVIDADEFAT').value = this.currentRow.QTHRATIVIDADEFAT;
        // Metadata.getInstance<TextInput>('form-tab-controle-apontamento-descricao-QTHORASNAOFATU').value = this.calculaHorasNaoFaturadas();
    }

    public setFilterGridHorasNaoFaturadas() {
        const gridHorasNaoFaturadas = Metadata.getInstance<TekGrid>('tek-grid-horas-cda-horas-nao-faturadas');

        gridHorasNaoFaturadas.datasource.setFilter({
            CDEMPRESA: this.currentRow.CDEMPRESA,
            CDFUNC: this.currentRow.CDFUNC,
            CDCLIENTE: this.currentRow.CDCLIENTE,
            DTSERVPREST: this.currentRow.DTSERVPREST,
            NRSERVPREST: this.currentRow.NRSERVPREST,
        });
    }

    public calculaHorasNaoFaturadas() {
        let totalHoras = '00:00';

        Http.post('/getHorasNaoFaturadas', {
            CDEMPRESA: this.currentRow.CDEMPRESA,
            CDFUNC: this.currentRow.CDFUNC,
            CDCLIENTE: this.currentRow.CDCLIENTE,
            DTSERVPREST: this.currentRow.DTSERVPREST,
            NRSERVPREST: this.currentRow.NRSERVPREST,
        }).then((response: IDictionary) => {
            if (response.data.data.length > 0) {
                totalHoras = this.somarHorasDeServicoNaoFaturado(response.data);
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });

        return totalHoras;
    }

    public somarHorasDeServicoNaoFaturado(dados: any[]): string {
        let totalHoras = '00:00';

        dados.forEach((item) => {
            const horaFormatada = this.corrigirFormatoHora(item.QTHORASNAOFATU);
            const horaSeparada = horaFormatada.split(':');
            const horas = parseInt(horaSeparada[0], 10);
            const minutos = parseInt(horaSeparada[1], 10);

            totalHoras = this.adicionarHoras(totalHoras, horas, minutos);
        });

        return totalHoras;
    }

    public adicionarHoras(horaExistente: string, horasParaAdicionar: number, minutosParaAdicionar: number): string {
        const [horaExistenteHoras, horaExistenteMinutos] = horaExistente.split(':');

        let horas = parseInt(horaExistenteHoras, 10);
        let minutos = parseInt(horaExistenteMinutos, 10);

        horas += horasParaAdicionar;
        minutos += minutosParaAdicionar;

        horas += Math.floor(minutos / 60);
        minutos %= 60;

        return `${String(horas).padStart(2, '0')}:${String(minutos).padStart(2, '0')}`;
    }

    public corrigirFormatoHora(hora: any) {
        // Remover quaisquer caracteres não numéricos
        let h = hora.replace(/\D/g, '');

        // Se a hora tiver menos de 4 dígitos, adicionar "0" à esquerda para obter um formato de 4 dígitos
        if (h.length < 4) {
            h = h.padStart(4, '0');
        }

        // Inserir dois pontos no formato HH:mm
        return `${h.slice(0, 2)}:${h.slice(2)}`;
    }

    public setFilterGridApontamentos() {
        const gridHorasNaoFaturadas = Metadata.getInstance<TekGrid>('tek-grid-apontamento');

        gridHorasNaoFaturadas.datasource.setFilter({
            NRAPROPAGENDAMENTO: this.currentRow.NRAPROPAGENDAMENTO,
        });
    }

    public alteraCamposModalApontamento({ component }: any) {
        const motivoSaida = Metadata.getInstance<Select>('form-modal-apontamento-IDMOTIVOSAIDA');
        const inicial = Metadata.getInstance<TextInput>('form-modal-apontamento-DTHRINICIOAPONTCDA');
        const fim = Metadata.getInstance<TextInput>('form-modal-apontamento-DTHRFIMCIOAPONTCDA');
        const observacao = Metadata.getInstance<Textarea>('form-modal-apontamento-DSOBSAPONTAMENTO');

        // opçao entrada -- campo motivo de saída  readonly
        if (component.value === '1') { // entrada
            motivoSaida.readonly = true;
            motivoSaida.removeValidation('required');
            motivoSaida.value = '';
            inicial.readonly = false;
            fim.readonly = false;
            observacao.readonly = false;
            observacao.removeValidation('required');
        } else if (component.value === '2') { // saída
            motivoSaida.readonly = false;
            motivoSaida.addValidation('required', {});
            inicial.readonly = false;
            fim.readonly = false;
            observacao.readonly = false;
            observacao.removeValidation('required');
        } else if (component.value === '3') { // falta
            motivoSaida.readonly = true;
            motivoSaida.removeValidation('required');
            motivoSaida.value = '';
            inicial.readonly = true;
            fim.readonly = true;
            observacao.readonly = false;
            observacao.addValidation('required', {});
        } else {
            motivoSaida.readonly = true;
            motivoSaida.removeValidation('required');
            inicial.readonly = true;
            fim.readonly = true;
            observacao.readonly = true;
            observacao.removeValidation('required');
        }
    }

    public alteraCampoObervacaoModalApontamento({ component }: any) {
        const observacao = Metadata.getInstance<Textarea>('form-modal-apontamento-DSOBSAPONTAMENTO');

        // opçao entrada -- campo motivo de saída  readonly
        if (component.value === '3') { // Saída Justificada
            observacao.readonly = false;
            observacao.addValidation('required', {});
        } else { // saída
            observacao.readonly = false;
            observacao.removeValidation('required');
            observacao.value = '';
        }
    }

    public calculaCampoTempo() {
        const inicio = Metadata.getInstance<Time>('form-modal-apontamento-DTHRINICIOAPONTCDA').value;
        const fim = Metadata.getInstance<Time>('form-modal-apontamento-DTHRFIMCIOAPONTCDA').value;

        if (inicio && fim) {
            const [horaEntrada, minutoEntrada] = inicio.split(':').map(Number);
            const [horaSaida, minutoSaida] = fim.split(':').map(Number);
            
            // Converter ambos os horários para minutos totais
            const totalMinutosEntrada = horaEntrada * 60 + minutoEntrada;
            const totalMinutosSaida = horaSaida * 60 + minutoSaida;
            
            // Calcular a diferença em minutos
            let diferencaMinutos = totalMinutosSaida - totalMinutosEntrada;
            
            // Caso o horário de saída seja menor que o de entrada (virada do dia)
            if (diferencaMinutos < 0) {
                diferencaMinutos += 24 * 60; // Adiciona 24 horas em minutos
            }
            
            // Converter de volta para horas e minutos
            const horas = Math.floor(diferencaMinutos / 60);
            const minutos = diferencaMinutos % 60;
            
            // Retornar a diferença no formato 'HH:mm'
            Metadata.getInstance<TextInput>('form-modal-apontamento-QTTEMPOAPONT').value = `${String(horas).padStart(2, '0')}${String(minutos).padStart(2, '0')}`;
        }
    }

    public converterHoraParaFormatoCompacto(hora: string): string {
        return hora.replace(':', '');
    }

    public converterFormatoCompactoParaHora(compacto: string): string {
        return `${compacto.slice(0, 2)}:${compacto.slice(2)}`;
    }

    public carregarDadosModalApontamento() {
        setTimeout(() => {
            Metadata.getInstance<Text>('title-modal-apontamento').text = 'Editar Apontamento';
            Metadata.getInstance<TextInput>('form-modal-apontamento-NRAPONTAMENTOAPROPAGEN').value = this.rowGridApontamento.NRAPONTAMENTOAPROPAGEN;
            Metadata.getInstance<Select>('form-modal-apontamento-IDTIPOAPONTAMENTO').value = this.rowGridApontamento.IDTIPOAPONTAMENTO;
            Metadata.getInstance<Select>('form-modal-apontamento-IDMOTIVOSAIDA').value = this.rowGridApontamento.IDMOTIVOSAIDA;
            Metadata.getInstance<Time>('form-modal-apontamento-DTHRINICIOAPONTCDA').value = this.converterFormatoCompactoParaHora(this.rowGridApontamento.DTHRINICIOAPONTCDA);
            Metadata.getInstance<Time>('form-modal-apontamento-DTHRFIMCIOAPONTCDA').value = this.converterFormatoCompactoParaHora(this.rowGridApontamento.DTHRFIMCIOAPONTCDA);
            Metadata.getInstance<TextInput>('form-modal-apontamento-QTTEMPOAPONT').value = this.converterHoraParaFormatoCompacto(this.rowGridApontamento.QTTEMPOAPONTADO);
            Metadata.getInstance<Textarea>('form-modal-apontamento-DSOBSAPONTAMENTO').value = this.rowGridApontamento.DSOBSAPONTAMENTO;
        }, 100);
    }

    public salvarApontamento() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-apontamento');
        const form = Metadata.getInstance<Form>('form-modal-apontamento');

        const params = {
            NRAPONTAMENTOAPROPAGEN: Metadata.getInstance<TextInput>('form-modal-apontamento-NRAPONTAMENTOAPROPAGEN').value,
            NRAPROPAGENDAMENTO: this.currentRow.NRAPROPAGENDAMENTO,
            IDTIPOAPONTAMENTO: Metadata.getInstance<Select>('form-modal-apontamento-IDTIPOAPONTAMENTO').value,
            IDMOTIVOSAIDA: Metadata.getInstance<Select>('form-modal-apontamento-IDMOTIVOSAIDA').value,
            DTHRINICIOAPONTCDA: this.converterHoraParaFormatoCompacto(Metadata.getInstance<Time>('form-modal-apontamento-DTHRINICIOAPONTCDA').value),
            DTHRFIMCIOAPONTCDA: this.converterHoraParaFormatoCompacto(Metadata.getInstance<Time>('form-modal-apontamento-DTHRFIMCIOAPONTCDA').value),
            QTTEMPOAPONT: this.converterFormatoCompactoParaHora(Metadata.getInstance<TextInput>('form-modal-apontamento-QTTEMPOAPONT').value),
            DSOBSAPONTAMENTO: Metadata.getInstance<Textarea>('form-modal-apontamento-DSOBSAPONTAMENTO').value,
        };

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/salvarApontamento', params).then(() => {
                Utilities.showSuccessMessage();
                grid.datasource.get();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                this.fecharModalApontamento();
                LoadingService.hide();
            });
        } else {
            Utilities.alertMessage('Preencha todos os campos obrigatórios', 'error');
        }
    }

    public dialogDeleteApontamento({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'ApropriacaoRecursosController.deleteApotamento');
    }

    public async deleteApotamento() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-apontamento');
        LoadingService.show();

        Http.post('/deleteApontamento', {
            NRAPONTAMENTOAPROPAGEN: grid.selectedRows.map((element: any) => element.NRAPONTAMENTOAPROPAGEN),
        }).then(() => {
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            grid.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }
}
